# -*- coding: utf-8 -*-
"""
Created on Wed Dec 25 15:46:32 2019

@author: tadahaya
"""
from .depro import Depro

__copyright__    = 'Copyright (C) 2020 MIZUNO Tadahaya'
__version__      = '0.0.11'
__license__      = 'MIT'
__author__       = 'MIZUNO Tadahaya'
__author_email__ = 'tadahaya@gmail.com'